/* File MSNPKT.C
 * Packet Driver interface
 *
 * Copyright (C) 1991, Trustees of Columbia University in the
 *  City of New York.  Permission is granted to any individual or
 *  institution to use, copy, or redistribute this software as long as
 *  it is not sold for profit and this copyright notice is retained.
 *
 * Written for MS-DOS Kermit by Joe R. Doupnik, 
 *  Utah State University, jrd@cc.usu.edu, jrd@usu.Bitnet
 *
 * Last edit
 * 6 Sept 1991
 */
#include "msntcp.h"
#include "msnlib.h"

#define BUFSIZE	3000			/* size of pkt receive buffer */

word	pkt_ip_type = 0x0008;		/* these are little endian values */
word	pkt_arp_type = 0x0608;
word	pkt_rarp_type = 0x3580;

word	pkt_ip_handle = -1;		/* -1 means invalid handle */
word	pkt_arp_handle = -1;
word	pkt_rarp_handle = -1;

int	pdversion, pdtype, pdnum, pdfunct;
extern	word pktdevclass;
extern	int pdinit(), pdinfo(), pdaccess(), pdclose();
extern	eth_address eth_addr;

static byte pktbuf[BUFSIZE] = {0}; /* linked list packet receive buffer */
static byte * pktbuf_read = pktbuf;
byte pktwnum = 0;		/* seq number for packets written to buffer */
byte pktrnum = 0;		/* seq number for packets read from buffer */
byte * pktbuf_wrote = &pktbuf[BUFSIZE - 4];


int
pkt_eth_init()
{
	
    	pkt_buf_wipe();		/* clean out and init receiver buffer */
	if (pdinit(pktbuf, eth_addr) == 0)
    		{
		outs("\r\nCannot attach to an Ethernet Packet Driver");
		return( 0 );
		}
	pkt_buf_wipe();
    				/* lets find out about the driver */
	if ((pdinfo(&pdversion, &pktdevclass, &pdtype, &pdnum, &pdfunct))
		== 0)
		{
		outs("\r\nCannot obtain Packet Driver information");
		return (0);
		}
	if (pdaccess(&pkt_arp_type, sizeof(pkt_arp_handle), &pkt_arp_handle)
		== 0)
 		{
		outs("\r\n\7Cannot access ARP type packets");
		return (0);
		}
	if (pdaccess(&pkt_ip_type, sizeof(pkt_ip_handle), &pkt_ip_handle)
		== 0)
 		{
		outs("\r\n\7Cannot access IP type packets");
		return (0);
		}
	if (pdaccess(&pkt_rarp_type, sizeof(pkt_rarp_handle), &pkt_rarp_handle)
		== 0)
 		{
		outs("\r\n\7Cannot access RARP type packets");
		return (0);
		}
	return (1);				/* say success */
}

int
pkt_release()
{
	register int status = 1;			/* assume success */

	if ( pktdevclass == PD_SLIP ) return (0);

    	if (pkt_arp_handle != -1)
		if (pdclose(pkt_arp_handle) == 0)
			{
			outs("\r\nERROR releasing Packet Driver for ARP");
			status = 0;
			}
		else pkt_arp_handle = -1;	/* handle is out of service */

	if (pkt_ip_handle != -1)
		if (pdclose(pkt_ip_handle) == 0)
			{
			outs("\r\nERROR releasing Packet Driver for IP");
			status = 0;
			}
		else pkt_ip_handle = -1;	/* handle is out of service */

	if (pkt_rarp_handle != -1)
		if (pdclose(pkt_rarp_handle) == 0)
			{
			outs("\r\nERROR releasing Packet Driver for RARP");
			status = 0;
			}
		else pkt_rarp_handle = -1;	/* handle is out of service */
	return (status);
}

/* Deliver pointer to start of packet (destination address) or NULL.
 * Simple linked list: byte flag, byte pkt seq number, int count, 
 * byte data[count] and so on.
 * Status on the link flag byte is
 * 0 = end of buffer (count has size to point to start of buffer)
 * 1 = this slot is free (unused)
 * 2 = this slot has an unread packet
 * 4 = this slot is allocated for a packet, packet not yet loaded into it
 * 8 = this slot has a packet which has been read once
 * A two byte count value follows the flag byte, for number of bytes in 
 * this slot. Pktbuf_read remembers the pointer to the last-read slot.
*/

void * pkt_received()
{
	register byte * p;
	register int i;

	p = pktbuf_read;			/* start with last read */
	for (i = 0; i < 2 * (BUFSIZE / (60 + 4)); i++)	/* 2 * max pkts */
		{
/* if link is:    end of buf    free     allocated   have read */
		if (*p == 0 || *p == 1 || *p == 4 || *p == 8)
			{
			p += 4 + *(word *)(p+2);   /* point at next link */
			continue;
			}
		if (*p == 2)			 /* ready to be read */
			{
			if (*(p+1) == pktrnum)	/* if this is the next pkt */
				{
 				pktbuf_read = p;	/* where we read */
				*p = 8;			/* mark as read */
				pktrnum++;		/* next one to read */
				return (p + 4);		/* return ptr to pkt*/
				}
			else
				{
				p += 4 + *(word *)(p+2); /* next link */
				i = 1;			/* restart counter */
				continue;
				}
			}
		else
			{
			pkt_buf_wipe();		/* emergency treatment */
			break;
			}
		}
	return (NULL);
}
	
void
pkt_buf_release(byte *p)	/* return a buffer to the pool */
{
	disable();		/* disallow interrupts here */
	p -= 4 + 6 + 6 + 2;	/* backup from IP header to link start */

	if (*p == 8)		/* if packet has been read */
		*p = 1;		/* mark link as freed */
	enable();
}

void
pkt_buf_wipe()					/* clear all buffers */
{
	disable();
	pktbuf[0] = 1;				/* flag first link as free */
	pktbuf[1] = 0;
	*(word *)&pktbuf[2] = BUFSIZE - 8; /* free space, size - two links */

	pktbuf[BUFSIZE - 4] = 0;		/* flag as end of buffer */
	pktbuf[BUFSIZE - 3] = 0;		/* pkt buffer seq number */
				/* count below points to start of buffer */
	*(word *)&pktbuf[BUFSIZE - 2] = - BUFSIZE; 
	pktbuf_read = pktbuf;				/* where last read */
	pktbuf_wrote = &pktbuf[BUFSIZE - 4];		/* where last wrote*/
	pktwnum = pktrnum = 0;		/* reset buffer sequence numbers */
	enable();
}

